use Test::More tests => 20;
use strict;
use DE_EPAGES::Core::API::AutoFollowUserAgent;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Core::API::File qw (GetFileContent CopyFile GetFileMD5Digest ExistsFile DeleteFile fsplit);
use DE_EPAGES::Presentation::API::Upload qw ( GetJSONImagesInfo GetJSONImageInfo );
use HTTP::Request;
use HTTP::Status qw( HTTP_OK );
use HTTP::Headers;

my $SHOPPATH = '/Shops/DemoShop';
my $TESTIMAGEFILENAME = 'test.jpg';
my $TESTTXTFILENAME = 'test.txt';

sub LoginToShop {
    my ($Shop) = @_;

    # login into demoshop mbo
    my $URL = $Shop->get('WebUrlAdminSSL');
    my $Browser = DE_EPAGES::Core::API::AutoFollowUserAgent->new;
    # $Browser->proxy(['http'], 'http://127.0.0.1:8888/');    # fiddler

    my $LoginResponse = $Browser->post(
        $URL,
        [
            'ChangeAction' => 'SaveLoginForm',
            'Login' => 'admin',
            'Password' => 'admin',
            'ViewAction' => 'MBO-ViewMBO'
        ]
    );

    is( $LoginResponse->code, HTTP_OK, 'Login 200 OK' ) or diag $LoginResponse->headers->as_string;

    $URL = $LoginResponse->header('content-base');

    return ($URL, $Browser);
}

sub UploadMultiPartFile {
    my ($Object, $SessionURL, $Browser, $ChangeAction, $UseMediaGallery, $UploadFile) = @_;
    $UseMediaGallery = $UseMediaGallery ? 1 : 0;
    my $TestDir = 'testdirectory';

    my $Headers = HTTP::Headers->new;
    $Headers->header(
        'handleashtml' => 1,
        'Content-Type' => 'multipart/form-data',
        'Accept' => 'text/html'
    );

    $SessionURL .= '?ViewAction=JSONViewResponse&ChangeAction='.$ChangeAction.'&UseMediaGallery='.$UseMediaGallery.'&ObjectID='.$Object->id;
    GetFileContent($UploadFile, { 'binary' => 1 });

    my $UploadResponse = $Browser->post(
        $SessionURL,
        'handleashtml' => 1,
        'Content-Type' => 'multipart/form-data',
        'Content' => [
            'method' => 'submit',
            'Name' => 'TestUpload',
            'Directory' => $TestDir,
            'NewFile' => [ $UploadFile ]
        ]
    );

    # send
    is( $UploadResponse->code, HTTP_OK, '200 OK' );

    # compare form binary data with file binary
    my $UploadDir;
    if($UseMediaGallery) {
        my $MediaGallery = $Object->getSite()->get('MediaGallery');
        $UploadDir = $MediaGallery->absolutePath('');
        ok(ExistsFile(join('/', $UploadDir, $TestDir, $UploadFile)), 'upload successful');
        my (undef, $MgtFileName, $MgtExt) = fsplit($UploadDir.'/'.$UploadFile);
        my $MgtFile = $MgtFileName.',mgt'.$MgtExt;
        ok(ExistsFile(join('/', $UploadDir, $TestDir, $MgtFile)), 'mgt conversion successful');
        DeleteFile(join('/', $UploadDir, $TestDir, $UploadFile));
        DeleteFile(join('/', $UploadDir, $TestDir, $MgtFile)) if ExistsFile(join('/', $UploadDir, $TestDir, $MgtFile));
    } else {
        $UploadDir = $Object->get('PublicPath');
        ok(ExistsFile(join('/', $UploadDir, $TestDir, $UploadFile)), 'upload successful');
        CheckFile($UploadFile, join('/', $UploadDir, $TestDir, $UploadFile));
        DeleteFile(join('/', $UploadDir, $TestDir, $UploadFile));
    }
}

sub UploadOctetFile {
    my ($Object, $SessionURL, $Browser, $ChangeAction, $UseMediaGallery, $UploadFile) = @_;
    $UseMediaGallery = $UseMediaGallery ? 1 : 0;

    my $Headers = HTTP::Headers->new;
    $Headers->header(
        'handleashtml' => 1,
        'content-disposition' => 'attachement; filename="'.$UploadFile.'";',
        'Content-Type' => 'application/octet-stream',
        'Accept' => 'text/html'
    );

    $SessionURL .= '?ViewAction=JSONViewResponse&ChangeAction='.$ChangeAction.'&UseMediaGallery='.$UseMediaGallery.'&ObjectID='.$Object->id;
    my $TestFileContent = GetFileContent($UploadFile, { 'binary' => 1 });
    my $UploadRequest = HTTP::Request->new(
        'POST',
        $SessionURL,
        $Headers,
        ${$TestFileContent}
    );
    my $UploadResponse = $Browser->request($UploadRequest);

    # send
    is( $UploadResponse->code, HTTP_OK, '200 OK' );

    # compare form binary data with file binary
    my $UploadDir;
    if($UseMediaGallery) {
        my $MediaGallery = $Object->getSite()->get('MediaGallery');
        $UploadDir = $MediaGallery->absolutePath('');
        ok(ExistsFile($UploadDir.'/'.$UploadFile), 'upload successful');
        my (undef, $MgtFileName, $MgtExt) = fsplit($UploadDir.'/'.$UploadFile);
        my $MgtFile = $MgtFileName.',mgt'.$MgtExt;
        ok(ExistsFile($UploadDir.'/'.$MgtFile), 'mgt conversion successful');
        DeleteFile($UploadDir.'/'.$UploadFile);
        DeleteFile($UploadDir.'/'.$MgtFile) if ExistsFile($UploadDir.'/'.$MgtFile);
    } else {
        $UploadDir = $Object->get('PublicPath');
        ok(ExistsFile($UploadDir.'/'.$UploadFile), 'upload successful');
        CheckFile($UploadFile, $UploadDir.'/'.$UploadFile);
        DeleteFile($UploadDir.'/'.$UploadFile);
    }
}

sub CheckFile() {
    my ($Source, $Target) = @_;
    cmp_ok(
        GetFileMD5Digest($Source),
        'eq',
        GetFileMD5Digest($Target),
        'file content md5 hash matches'
    );
}

sub TestUploads {
    my ($Shop) = @_;

    my ($SessionURL, $Browser) = LoginToShop($Shop);
    UploadOctetFile($Shop, $SessionURL, $Browser, 'UploadFiles', 0, $TESTTXTFILENAME);
    UploadMultiPartFile($Shop, $SessionURL, $Browser, 'UploadFiles', 0, $TESTTXTFILENAME);
    UploadOctetFile($Shop, $SessionURL, $Browser, 'UploadFilesToMediaGallery', 1, $TESTIMAGEFILENAME);
    UploadMultiPartFile($Shop, $SessionURL, $Browser, 'UploadFilesToMediaGallery', 1, $TESTIMAGEFILENAME);
    my $ShopPublicPath = $Shop->get('PublicPath');
    my $Image1 = $ShopPublicPath . '/' . $TESTIMAGEFILENAME;
    my $Image2 = $ShopPublicPath . '/' . $TESTIMAGEFILENAME . '_2.jpg';
    CopyFile($TESTIMAGEFILENAME, $Image1);
    CopyFile($TESTIMAGEFILENAME, $Image2);
    my $hJsonImagesData = GetJSONImagesInfo($ShopPublicPath, [ $Image1, $Image2 ]);
    ok(defined $hJsonImagesData->{'imageInfo'}, 'GetJSONImagesInfo imageInfo');
    ok(defined $hJsonImagesData->{'uploadedObjectFiles'}, 'GetJSONImagesInfo uploadedObjectFiles');
    cmp_ok(scalar(@{$hJsonImagesData->{'imageInfo'}}), '==', 2, 'GetJSONImageInfos imageInfo 2 images');
    cmp_ok(scalar(@{$hJsonImagesData->{'uploadedObjectFiles'}}), '==', 2, 'GetJSONImageInfos uploadedObjectFiles 2 images');
    my $hJsonImageData = GetJSONImageInfo($ShopPublicPath, $Image1);
    ok(defined $hJsonImageData->{'imageInfo'}, 'GetJSONImageInfo imageInfo');
    ok(defined $hJsonImageData->{'uploadedObjectFile'}, 'GetJSONImageInfo uploadedObjectFile');
    DeleteFile($Image1);
    DeleteFile($Image2);
}

sub Main {
    if( ExistsObjectByPath($SHOPPATH) ) {
        my $Shop = LoadObjectByPath($SHOPPATH);
        ok( $Shop->id, 'load demoshop ' . $Shop->id );
        TestUploads($Shop);
    } else {
        SKIP: {
            skip 'no DemoShop installed', 20;
        };
    }
    return;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
