﻿#========================================================================================
# §package DE_EPAGES::RemoteSearch::t::ViewFacetedSearch
# §state   private
#----------------------------------------------------------------------------------------
# §description Tests FacetedSearchProducts with FakeRequest
#              Needs a Store with DemoShop
#========================================================================================
package DE_EPAGES::RemoteSearch::t::ViewFacetedSearch;
use base Test::Class;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::WebInterface::API::FakeRequest;
use JSON;
use Test::More;
use utf8;

sub setUp : Test(startup) {
    my $self = shift;
    plan skip_all => 'no demoshop installed'     unless ( ExistsObjectByPath('/Shops/DemoShop') );
    plan skip all => 'remote search not enabled' unless ( LoadRootObject->get('RemoteSearchEnabled') );

    $self->shop( LoadObjectByPath('/Shops/DemoShop') );
    plan skip_all => 'DemoShop did not initialize remotesearch'
      unless ( defined $self->shop->get('RemoteProductSearchIndex')
        && $self->shop->get('RemoteProductSearchIndex')->usableAtStorefront );
}

sub testSimple : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL = BuildShopUrl( $self->shop, { 'ViewAction' => 'FacetedSearchProducts', 'SearchString' => 'Jack' }, {} );
    my $Response = $FakeRequest->get($URL);

    ok( $Response->is_success, 'ViewAction with searchstring works' ) or diag( $Response->status_line );
    ok( $Response->decoded_content() =~ m/FacetName/, 'Found facets with searchstring' );
}

sub testUmlauts : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL = BuildShopUrl( $self->shop, {
        'ViewAction' => 'FacetedSearchProducts',
        'SearchString' => 'Schmusebär',
        'Locale' => 'de_DE'
    }, {} );
    my $Response = $FakeRequest->get($URL);

    ok( $Response->is_success, 'ViewAction with umlauts works' ) or diag( $Response->status_line );
    ok( $Response->decoded_content() =~ m/FacetName/, 'Found facets with umlauts' );
}

sub testEmptyString : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL      = BuildShopUrl( $self->shop, { 'ViewAction' => 'FacetedSearchProducts', 'SearchString' => '' }, {} );
    my $Response = $FakeRequest->get($URL);

    ok( $Response->is_success, 'ViewAction without searchstring works' ) or diag( $Response->status_line );
    ok( $Response->decoded_content() =~ m/FacetName/, 'Found facets without searchstring' );
}

sub testFacets : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $CurrencyID  = GetInfoLocale( $self->shop->get('SiteLocale')->localeID )->{'CurrencyID'};
    my $URL         = BuildShopUrl(
        $self->shop,
        {
            'ViewAction'              => 'FacetedSearchProducts',
            'SearchString'            => '',
            'FacetValue_Manufacturer' => [ 'Berghaus', 'Eureka', 'Deuter' ],
            'FacetValue_IsNew'        => '1',
            'CurrencyID'              => [ $CurrencyID, $CurrencyID ],
            'FacetRange_ListPrice' =>
              [ $self->shop->get('SiteLocale')->formatMoney(20), $self->shop->get('SiteLocale')->formatMoney(250) ],
        },
        {}
    );
    my $Response = $FakeRequest->get($URL);

    ok( $Response->is_success, 'ViewAction with facets works. ' ) or diag($URL);
    ok( $Response->decoded_content() =~ m/FacetName/, 'Found facets with facet input' ) or diag($URL);
}

sub testOtherCurrency : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    # get second locale in list
    my $Locale      = $self->shop->get('SiteLocales')->[1];
    my $CurrencyID  = $Locale->get('Currency')->{CurrencyID},
    my $Symbol      = $Locale->get('Currency')->{Symbol};

    my $URL         = BuildShopUrl(
        $self->shop,
        {
            'ViewAction'              => 'FacetedSearchProducts',
            'SearchString'            => '',
            'Currency'                => $CurrencyID,
        },
        {}
    );
    my $Response = $FakeRequest->get($URL);
    ok( $Response->is_success, 'ViewAction with other currency works. ' ) or diag($URL);
    ok( $Response->decoded_content() =~ qr/$Symbol/, 'SF: found a '.$CurrencyID ) or diag($URL);
}

sub testOtherCurrencyMobile : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    # get second locale in list
    my $Locale      = $self->shop->get('SiteLocales')->[1];
    my $CurrencyID  = $Locale->get('Currency')->{CurrencyID},
    my $Symbol      = $Locale->get('Currency')->{Symbol};

    my $URL         = BuildShopUrl(
        $self->shop,
        {
            'ViewAction'              => 'MobileSF-DetailFacetedSearchProducts',
            'SearchString'            => '',
            'Currency'                => $CurrencyID,
        },
        {
            'Type' => 'mobile'
        }
    );
    my $Response = $FakeRequest->get($URL);
    ok( $Response->is_success, 'MobileSF ViewAction with other currency works. ' ) or diag($URL);
    ok( $Response->decoded_content() =~ qr/$Symbol/, 'MobileSF: found a '.$CurrencyID ) or diag($URL);
}

sub testFacetsWithInvalidInput : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $CurrencyID  = GetInfoLocale( $self->shop->get('SiteLocale')->localeID )->{'CurrencyID'};
    my $URL         = BuildShopUrl(
        $self->shop,
        {
            'ViewAction'              => 'FacetedSearchProducts',
            'SearchString'            => '',
            'FacetValue_Manufacturer' => [ 'Berghaus', 'Eureka', 'Deuter' ],
            'FacetValue_IsNew'        => '1',
            'CurrencyID'              => [ $CurrencyID, $CurrencyID ],
            'FacetRange_ListPrice' =>
              [ 'x', 'y' ],
        },
        {}
    );
    my $Response = $FakeRequest->get($URL);

    ok( $Response->is_success, 'ViewAction with invalid input works. ' ) or diag($URL);
    ok( $Response->decoded_content() =~ m/FacetName/, 'Found facets with facet input' ) or diag($URL);
}

sub testFacetsWithMultiCategoryInput : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL         = BuildShopUrl(
        $self->shop,
        {
            'ViewAction'              => 'FacetedSearchProducts',
            'SearchString'            => '',
            'FacetValue_CategoryID'   => [ $self->shop->child('Categories')->child('Shoes')->id ,$self->shop->child('Categories')->child('Jackets')->id ],
        },
        {}
    );
    my $Response = $FakeRequest->get($URL);

    ok( $Response->is_success, 'ViewAction with multiple CategoryIDs works. ' ) or diag($URL);
    ok( $Response->decoded_content() =~ m/FacetName/, 'Found facets with facet input' ) or diag($URL);
}

sub shop {
    my $self = shift;
    my ($shop) = @_;
    return $self->{'shop'} unless ( defined($shop) );
    $self->{'shop'} = $shop;
}

RunScript(
    Sub => sub {
        RunOnStore(
            'Store' => GetTestSite(),
            'Sub'   => sub {
                Test::Class->runtests;
            }
        );
    }
);
